module Md2site
  #
  # HTMLファイル分割クラス
  class HTMLUtils
    require "md2site/nkfguess"

    #
    # 初期化
    #
    # @param htmlfname [String] 分割対象HTMLファイル名
    # @param mes [Messagex] Messagexクラスのインスタンス
    # @return [void]
    def initialize(htmlfname, mes)
      @htmlfname = htmlfname
      @mes = mes
    end

    #
    # 1行解析
    #
    # @param state [Symbol] 分割対象HTMLファイル名
    # @param line [String] 解析対象行
    # @return [Symbol] 構文解析上の次の状態
    def parse_line(state, line)
      event = get_event(line)

      @mes.output_debug("#{@lineno}|#{state}|#{event}|#{line}")

      case state
      when :IN_PART0
        next_state = state_in_part0(event, line)
      when :IN_PART0_CONTENTS
        next_state = state_in_part0_contents(event, line)
      when :IN_PART1
        next_state = state_in_part1(event, line)
      when :IN_PART2
        next_state = state_in_part2(event, line)
      else
        @mes.output_fatal("state=#{state}")
        @mes.output_fatal("event=#{event}")
        raise
      end
      next_state
    end

    #
    # HTMLファイル分割
    #
    # @return [Array] 第0要素:HTML先頭部 第1要素:HTML中間部 第2要素:HTML末尾部
    def divide_html
      part_struct = Struct.new(:lines, :status)
      @part0 = part_struct.new([], false)
      @part1 = part_struct.new([], false)
      @part2 = part_struct.new([], false)
      @div_body_main = nil
      @div_cse = nil

      encoding = Nkfguess.guess_file(@htmlfname, @mes)
      mode = "r:#{encoding}"
      f = @mes.exc_file_gets(@htmlfname) { File.open(@htmlfname, mode) }

      @lines = []
      @lineno = 0
      state = :IN_PART0
      while (line = @mes.exc_file_gets(@htmlfname) { f.gets })
        state = parse_line(state, line)
      end
      @part2.lines << @lines

      @mes.exc_file_close(@htmlfname) { f.close }

      [@part0.lines.flatten.join("\n"),
       @part1.lines.flatten.join("\n"),
       @part2.lines.flatten.join("\n")]
    end

    private

    #
    # 解析対象行からトークン取得
    #
    # @param line [String] 解析対象行
    # @return [Symbol] トークン種別
    def get_event(line)
      line.chomp!
      @lineno += 1
      if line.index("CONTENTS BEGIN")
        event = :CONTENTS_BEGIN
      elsif line.index("CONTENTS END")
        event = :CONTENTS_END
      elsif line.index("<div")
        if line.index(%Q(id="body_main"))
          event = :ID_BODY_MAIN
        elsif line.index(%Q(id="cse"))
          event = :ID_CSE
        else
          event = :ELSE
        end
      elsif /^\s*$/.match?(line)
        event = :WHITE_SPACE
      else
        event = :ELSE
      end
      event
    end

    #
    # part0部のCONTENTS_ENDイベント処理
    #
    # @param line [String] 解析対象行
    # @return [Symbol] 構文解析上の次の状態
    def state_in_part0_contents_end(line)
      @part0.lines << @lines
      if !@div_body_main.nil? && !@div_cse.nil?
        @part0.status = true
      end
      @lines = []
      @lines << line
      raise unless @div_body_main
      raise unless @div_cse
      :IN_PART2
    end

    #
    # part0部のイベント処理
    #
    # @param event [Symbol] トークン種別
    # @param line [String] 解析対象行
    # @return [Symbol] 構文解析上の次の状態
    def state_in_part0(event, line)
      case event
      when :CONTENTS_BEGIN
        @part0.lines << @lines
        if !@div_body_main.nil? && !@div_cse.nil?
          @part0.status = true
        end
        @lines = []
        @lines << line
        next_state = :IN_PART0_CONTENTS
      when :CONTENTS_END
        next_state = state_in_part0_contents_end(line)
      when :ID_BODY_MAIN
        @lines << line
        @div_body_main ||= line
        next_state = :IN_PART1
      when :ID_CSE
        @lines << line
        @div_cse ||= line
        next_state = :IN_PART1
      when :WHITE_SPACE
        @lines << line
        next_state = :IN_PART1
      when :ELSE
        @lines << line
        next_state = :IN_PART1
      else
        raise
      end
      next_state
    end

    #
    # part0_contents部のcontents_endイベント処理
    #
    # @param line [String] 解析対象行
    # @return [Symbol] 構文解析上の次の状態
    def state_in_part0_contents_contents_end(line)
      @part1.lines << @lines
      if !@div_body_main.nil? && !@div_cse.nil?
        @part1.status = true
      end
      @lines = []
      @lines << line
      raise unless @div_body_main
      raise unless @div_cse
      :IN_PART2
    end

    #
    # part0_contents部のイベント処理
    #
    # @param event [Symbol] トークン種別
    # @param line [String] 解析対象行
    # @return [Symbol] 構文解析上の次の状態
    def state_in_part0_contents(event, line)
      case event
      when :CONTENTS_BEGIN
        @lines << line
        next_state = :IN_PART0_CONTENTS
      when :CONTENTS_END
        next_state = state_in_part0_contents_contents_end(line)
      when :ID_BODY_MAIN
        @lines << line
        @div_body_main = line
        next_state = :IN_PART0_CONTENTS
      when :ID_CSE
        @lines << line
        @div_cse = line
        next_state = :IN_PART0_CONTENTS
      when :WHITE_SPACE
        @lines << line
        next_state = :IN_PART0_CONTENTS
      when :ELSE
        @part0.lines << @lines
        if !@div_body_main.nil? && !@div_cse.nil?
          @part0.status = true
        end
        @lines = []
        @lines << line
        next_state = :IN_PART1
      else
        raise
      end
      next_state
    end

    #
    # part1部のcontents_endイベント処理
    #
    # @param line [String] 解析対象行
    # @return [Symbol] 構文解析上の次の状態
    def state_in_part1_contents_end(line)
      @part1.lines << @lines
      if !@div_body_main.nil? && !@div_cse.nil?
        @part1.status = true
      end
      @lines = []
      @lines << line
      raise unless @div_body_main
      raise unless @div_cse
      :IN_PART2
    end

    #
    # part1部のイベント処理
    #
    # @param event [Symbol] トークン種別
    # @param line [String] 解析対象行
    # @return [Symbol] 構文解析上の次の状態
    def state_in_part1(event, line)
      case event
      when :CONTENTS_BEGIN
        @part0.lines << @lines
        @lines = []
        @lines << line
        next_state = :IN_PART0_CONTENTS
      when :CONTENTS_END
        next_state = state_in_part1_contents_end(line)
      when :ID_BODY_MAIN
        @div_body_main = line
        @part0.lines << line

        next_state = :IN_PART1
      when :ID_CSE
        @div_cse = line
        @part0.lines << line

        next_state = :IN_PART1
      when :WHITE_SPACE
        @lines << line
        next_state = :IN_PART1
      when :ELSE
        @lines << line
        next_state = :IN_PART1
      else
        @mes.output_fatal("state=:IN_PART1")
        @mes.output_fatal("event=#{event}|")
        raise
      end
      next_state
    end

    #
    # part2部のイベント処理
    #
    # @param event [Symbol] トークン種別
    # @param line [String] 解析対象行
    # @return [Symbol] 構文解析上の次の状態
    def state_in_part2(event, line)
      @lines << line

      :IN_PART2
    end
  end
end
